#include "const.h"

// Definition of constant ----------------------------------------------------
const double PI     = 3.1415926535897932e0;     //! pi 
const double D2R    = (PI/180.e0);              //! deg to rad 
const double R2D    = (180.e0/PI);              //! rad to deg 
const double CLIGHT = 299792458.e0;             //! speed of light (m/s) 
const double SC2RAD = 3.1415926535898e0;        //! semi-circle to radian (IS-GPS) 
const double AU     = 149597870691.e0;          //! 1 AU (m) 
const double AS2R   = (D2R/3600.e0);            //! arc sec to radian 

const double OMGE   = 7.2921151467e-5;          //! earth angular velocity (IS-GPS) (rad/s)
const double RE_WGS84 = 6378137.e0;             //! earth semimajor axis (WGS84) (m) 
const double FE_WGS84 = (1.e0/298.257223563e0); //! earth flattening (WGS84) 

const double HION     = 350000.e0;              //! ionosphere height (m) 


const double FREQL1    = 1.57542e9;             //! L1/E1  frequency (Hz)
const double FREQL2    = 1.22760e9;             //! L2     frequency (Hz)
const double FREQL5    = 1.17645e9;             //! L5/E5a frequency (Hz)
const double FREQL6    = 1.27875e9;             //! E6/LEX frequency (Hz)
const double FREQE5b   = 1.20714e9;             //! E5b    frequency (Hz)
const double FREQE5ab  = 1.191795e9;            //! E5a+b  frequency (Hz)
const double FREQs     = 2.492028e9;            //! S      frequency (Hz)
const double FREQ1_GLO = 1.60200e9;             //! GLONASS G1 base frequency (Hz)
const double DFRQ1_GLO = 0.56250e6;             //! GLONASS G1 bias frequency (Hz/n)
const double FREQ2_GLO = 1.24600e9;             //! GLONASS G2 base frequency (Hz)
const double DFRQ2_GLO = 0.43750e6;             //! GLONASS G2 bias frequency (Hz/n)
const double FREQ1a_GLO = 1.600995E9;           // GLONASS G1a frequency (Hz) 
const double FREQ2a_GLO = 1.248060E9;           // GLONASS G2a frequency (Hz) 
const double FREQ3_GLO = 1.202025e9;            //! GLONASS G3 frequency (Hz)
const double FREQ1_CMP = 1.561098e9;            //! BeiDou B1 frequency (Hz)
const double FREQ2_CMP = 1.20714e9;             //! BeiDou B2 frequency (Hz)
const double FREQ3_CMP = 1.26852e9;             //! BeiDou B3 frequency (Hz)

const double EFACT_GPS = 1.e0;                  //! error factor: GPS
const double EFACT_GLO = 1.5e0;                 //! error factor: GLONASS
const double EFACT_GAL = 1.e0;                  //! error factor: Galileo
const double EFACT_QZS = 1.e0;                  //! error factor: QZSS
const double EFACT_CMP = 1.e0;                  //! error factor: BeiDou
const double EFACT_IRN = 1.5e0;                 //! error factor: IRNSS
const double EFACT_SBS = 3.e0;                  //! error factor: SBAS



const double DTTOL       = 0.015e0;             //! tolerance of time difference (s) 
const double MAXDTOE     = 7200.e0;             //! max time difference to GPS Toe (s) 
const double MAXDTOE_QZS = 7200.e0;             //! max time difference to QZSS Toe (s) 
const double MAXDTOE_GAL = 14400.e0;            //! max time difference to Galileo Toe (s) 
const double MAXDTOE_CMP = 21600.e0;            //! max time difference to BeiDou Toe (s) 
const double MAXDTOE_GLO = 1800.e0;             //! max time difference to GLONASS Toe (s) 
const double MAXDTOE_SBS = 360.e0;              //! max time difference to SBAS Toe (s) 
const double MAXDTOE_S   = 86400.e0;            //! max time difference to ephem toe (s) for other 
const double MAXGDOP     = 300.e0;              //! max GDOP 

const double INT_SWAP_TRAC = 86400.e0;          //! swap interval of trace file (s) 
const double INT_SWAP_STAT = 86400.e0;          //! swap interval of solution status file (s) 


const double  MAXSBSAGEF = 30.e0;               //! max age of SBAS fast correction (s) 
const double  MAXSBSAGEL = 1800.e0;             //! max age of SBAS long term corr (s) 

const double RNX2VER = 2.10e0;                  //! RINEX ver.2 default output version 
const double RNX3VER = 3.00e0;                  //! RINEX ver.3 default output version 


const char COMMENTH[2] = "%";                   //! comment line indicator for solution 
const char MSG_DISCONN[13] = "$_DISCONNECT";    //! disconnect message

const double P2_5  = 0.03125e0;                 //! 2^-5
const double P2_6  = 0.015625e0;                //! 2^-6
const double P2_11 = 4.882812500000000e-04;     //! 2^-11
const double P2_15 = 3.051757812500000e-05;     //! 2^-15
const double P2_17 = 7.629394531250000e-06;     //! 2^-17
const double P2_19 = 1.907348632812500e-06;     //! 2^-19
const double P2_20 = 9.536743164062500e-07;     //! 2^-20
const double P2_21 = 4.768371582031250e-07;     //! 2^-21
const double P2_23 = 1.192092895507810e-07;     //! 2^-23
const double P2_24 = 5.960464477539063e-08;     //! 2^-24
const double P2_27 = 7.450580596923828e-09;     //! 2^-27
const double P2_29 = 1.862645149230957e-09;     //! 2^-29
const double P2_30 = 9.313225746154785e-10;     //! 2^-30
const double P2_31 = 4.656612873077393e-10;     //! 2^-31
const double P2_32 = 2.328306436538696e-10;     //! 2^-32
const double P2_33 = 1.164153218269348e-10;     //! 2^-33
const double P2_35 = 2.910383045673370e-11;     //! 2^-35
const double P2_38 = 3.637978807091710e-12;     //! 2^-38
const double P2_39 = 1.818989403545856e-12;     //! 2^-39
const double P2_40 = 9.094947017729280e-13;     //! 2^-40
const double P2_43 = 1.136868377216160e-13;     //! 2^-43
const double P2_48 = 3.552713678800501e-15;     //! 2^-48
const double P2_50 = 8.881784197001252e-16;     //! 2^-50
const double P2_55 = 2.775557561562891e-17;     //! 2^-55

//! rtkcmn.f90 ----------------------------------------------------------------
const double MAX_VAR_EPH = 9e4;                //! max variance eph to reject satellite (m^2) 
const double gpst0[6]={1980,1, 6,0,0,0};       //! gps time reference 
const double gst0[6] ={1999,8,22,0,0,0};       //! galileo system time reference 
const double bdt0[6] ={2006,1, 1,0,0,0};       //! beidou time reference 

const double lam_carr[MAXFREQ]={               //! carrier wave length (m)
    CLIGHT/FREQL1,CLIGHT/FREQL2,CLIGHT/FREQL5,CLIGHT/FREQL6,CLIGHT/FREQE5b,
    CLIGHT/FREQE5ab,CLIGHT/FREQs};

//! leap seconds (y,m,d,h,m,s,utc-gpst) 
const double leaps[19][7]={
    {2017,1,1,0,0,0,-18},
    {2015,7,1,0,0,0,-17},
    {2012,7,1,0,0,0,-16},
    {2009,1,1,0,0,0,-15},
    {2006,1,1,0,0,0,-14},
    {1999,1,1,0,0,0,-13},
    {1997,7,1,0,0,0,-12},
    {1996,1,1,0,0,0,-11},
    {1994,7,1,0,0,0,-10},
    {1993,7,1,0,0,0, -9},
    {1992,7,1,0,0,0, -8},
    {1991,1,1,0,0,0, -7},
    {1990,1,1,0,0,0, -6},
    {1988,1,1,0,0,0, -5},
    {1985,7,1,0,0,0, -4},
    {1983,7,1,0,0,0, -3},
    {1982,7,1,0,0,0, -2},
    {1981,7,1,0,0,0, -1},
    {0   ,0,0,0,0,0,  0}
    };

//! observation code strings 
const char obscodes[60][3]={
    {"  "},{"1C"},{"1P"},{"1W"},{"1Y"}, {"1M"},{"1N"},{"1S"},{"1L"},{"1E"},  //!  0- 9 
    {"1A"},{"1B"},{"1X"},{"1Z"},{"2C"}, {"2D"},{"2S"},{"2L"},{"2X"},{"2P"},  //! 10-19 
    {"2W"},{"2Y"},{"2M"},{"2N"},{"5I"}, {"5Q"},{"5X"},{"7I"},{"7Q"},{"7X"},  //! 20-29 
    {"6A"},{"6B"},{"6C"},{"6X"},{"6Z"}, {"6S"},{"6L"},{"8L"},{"8Q"},{"8X"},  //! 30-39 
    {"2I"},{"2Q"},{"6I"},{"6Q"},{"3I"}, {"3Q"},{"3X"},{"1I"},{"1Q"},{"5A"},  //! 40-49 
    {"5B"},{"5C"},{"9A"},{"9B"},{"9C"}, {"9X"},{"  "},{"  "},{"  "},{"  "}   //! 50-59
};

//! 1:L1/E1, 2:L2/B1, 3:L5/E5a/L3, 4:L6/LEX/B3, 5:E5b/B2, 6:E5(a+b), 7:S
const int obsfreqs[60]={
    0, 1, 1, 1, 1,  1, 1, 1, 1, 1,  //!  0- 9
    1, 1, 1, 1, 2,  2, 2, 2, 2, 2,  //! 10-19
    2, 2, 2, 2, 3,  3, 3, 5, 5, 5,  //! 20-29
    4, 4, 4, 4, 4,  4, 4, 6, 6, 6,  //! 30-39
    2, 2, 4, 4, 3,  3, 3, 1, 1, 3,  //! 40-49
    3, 3, 7, 7, 7,  7, 0, 0, 0, 0   //! 50-59
};

//! code priority table 
//! L1/E1; L2/B1; L5/E5a/L3; L6/LEX/B3; E5b/B2; E5(a+b); S
const char codepris[7][MAXFREQ][17]={
    {"CPYWMNSL  ","PYWCMNDSLX","IQX       ","          ","          ","          ","          "},  //! GPS
    {"PC        ","PC        ","IQX       ","          ","          ","          ","          "},  //! GLO
    {"CABXZ     ","          ","IQX       ","ABCXZ     ","IQX       ","IQX       ","          "},  //! GAL
    {"CSLXZ     ","SLX       ","IQX       ","SLX       ","          ","          ","          "},  //! QZS
    {"C         ","          ","IQX       ","          ","          ","          ","          "},  //! SBS
    {"IQX       ","IQX       ","IQX       ","IQX       ","IQX       ","          ","          "},  //! BDS
    {"          ","          ","ABCX      ","          ","          ","          ","ABCX      "}   //! IRN
};

//! ephemeris.f90 -------------------------------------------------------------
const double RE_GLO = 6378136.e0;              //! radius of earth (m)            ref (2) 
const double MU_GPS = 3.9860050e14;            //! gravitational constant         ref (1) 
const double MU_GLO = 3.9860044e14;            //! gravitational constant         ref (2) 
const double MU_GAL = 3.986004418e14;          //! earth gravitational constant   ref (7) 
const double MU_CMP = 3.986004418e14;          //! earth gravitational constant   ref (9) 
const double J2_GLO = 1.0826257e-3;            //! 2nd zonal harmonic of geopot   ref (2) 

const double OMGE_GLO = 7.292115e-5;           //! earth angular velocity (rad/s) ref (2) 
const double OMGE_GAL = 7.2921151467e-5;       //! earth angular velocity (rad/s) ref (7) 
const double OMGE_CMP = 7.292115e-5;           //! earth angular velocity (rad/s) ref (9) 
const double SIN_5 = -0.0871557427476582e0;    //! dsin(-5.e0 deg) 
const double COS_5 =  0.9961946980917456e0;    //! dcos(-5.e0 deg) 

const double ERREPH_GLO  = 5.e0;               //! error of glonass ephemeris (m) 
const double TSTEP       = 60.e0;              //! integration step glonass ephemeris (s) 
const double RTOL_KEPLER = 1e-13;              //! relative tolerance for Kepler equation 

const double DEFURASSR  = 0.15e0;              //! default accurary of ssr corr (m) 
const double MAXECORSSR = 10.e0;               //! max orbit correction of ssr (m) 
const double MAXCCORSSR = (1e-6*CLIGHT);       //! max clock correction of ssr (m) 
const double MAXAGESSR  = 90.e0;               //! max age of ssr orbit and clock (s) 
const double MAXAGESSR_HRCLK = 10.e0;          //! max age of ssr high-rate clock (s) 
const double STD_BRDCCLK = 30.e0;              //! error of broadcast clock (m) 
const double STD_GAL_NAPA = 500.e0;            //! error of galileo ephemeris for NAPA (m) 


const int eph_sel[6]={0,0,1,0,0,0};            //! GPS,GLO,GAL,QZS,BDS,SBS (ephemeris selections)

//! pntpos.f90 ----------------------------------------------------------------
const double ERR_ION   = 5.e0 ;                //! ionospheric delay std (m) 
const double ERR_TROP  = 3.e0;                 //! tropspheric delay std (m) 
const double ERR_SAAS  = 0.3e0;                //! saastamoinen model error std (m) 
const double ERR_BRDCI = 0.5e0;                //! broadcast iono model error factor 
const double ERR_CBIAS = 0.3e0;                //! code bias error std (m) 
const double REL_HUMI  = 0.7e0;                //! relative humidity for saastamoinen model 

//! rinex.f90 -----------------------------------------------------------------
const char systyps[8]  = "GREJSCI";            //! satellite system codes 
const char obstyps[5]  = "CLDS";               //! obs type codes 
const char frqtyps[8]  = "1256789";            //! frequency codes 
const double ura_eph[16] = {                   //! ura values (ref [3] 20.3.3.3.1.1) 
    2.4,3.4,4.85,6.85,9.65,13.65,24.0,48.0,96.0,
    192.0,384.0,768.0,1536.0,3072.0,6144.0,0.00};
const double ura_nominal[16] = {               //! ura nominal values 
    2.0,2.8,4.0,5.7,8.0,11.3,16.0,32.0,64.0,
    128.0,256.0,512.0,1024.0,2048.0,4096.0,8192.0};

